import type { CrudManagerConfig } from "@bryntum/scheduler";
import { AjaxStore } from "@bryntum/scheduler";
import { AdditionalStoresEnum } from "../../eventsEnum"
import { Toast } from "@bryntum/scheduler";

export default (scheduleId: number): Partial<CrudManagerConfig> => {
    const wishTypes = new AjaxStore({ id: AdditionalStoresEnum.wishTypes })
    const scheduleActions = new AjaxStore({ id: AdditionalStoresEnum.scheduleActions, fields: ['name', 'text', 'method', 'route',], })
    const rooms = new AjaxStore({ id: AdditionalStoresEnum.rooms })
    const users = new AjaxStore({ id: AdditionalStoresEnum.users })
    const timeRanges = new AjaxStore({ id: AdditionalStoresEnum.timeRanges })
    return {
        stores: [users, rooms, wishTypes, scheduleActions, timeRanges],
        autoLoad: true,
        autoSync: true,
        writeAllFields: true,
        transport: {
            load: {
                url: `http://localhost:3030/mocks/response.json`,
                method: 'GET',
            },
            sync: {
                url: `http://localhost:3030/mocks/response.json`,
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                }
            }
        },
        validateResponse: true, // TODO: Not in PRODUCTION!
        onRequestFail: async ({ source, requestType, response, responseText, responseOptions }) => {
            source.revertChanges() // Stop retrying

            console.error("requestType", requestType)
            console.error("responseText", responseText)
            console.error("response", response)
            const responseTextObj = JSON.parse(responseText)
            let message = '';
            if ('message' in responseTextObj && 'errors' in responseTextObj) {
                message = `${responseTextObj.message}: ${JSON.stringify(responseTextObj.errors)}`
            }
            else if ('message' in responseTextObj) {
                message = `${responseTextObj.message}`
            } else {
                message = JSON.stringify(responseTextObj)
            }
            Toast.show({
                html: message,
                color: 'b-red',
                style: 'color:white',
                timeout: 10000
            });
        },
    }
}
