import type { AjaxStore, EventEditConfig, } from "@bryntum/scheduler";
import { DateHelper } from "@bryntum/scheduler";
import { CONSTS } from "./utils";

export default (wishTypes: AjaxStore, rooms: AjaxStore, users: AjaxStore, showOnlyWish: boolean, rowKey: string): Partial<EventEditConfig> => {
    return {
        timeFormat: "LT",
        editorConfig: {
            width: '32em'
        },
        items: {
            eventType: {
                type: 'combo',
                name: 'eventType',
                label: 'Type',
                weight: 2,
                disabled: showOnlyWish,
                items: [CONSTS.wish, CONSTS.shift],
                listeners: {
                    change: ({ source, value }) => {
                        // toggle visibility of widgets belonging to eventTypes
                        source.owner.items.forEach(widget => {
                            if (widget.dataset && widget.dataset.eventType) {
                                widget.hidden = widget.dataset.eventType !== value;
                                widget.required = widget.dataset.eventType === value;
                            }
                        });
                        const startOffset = value === CONSTS.shift ? 8 : 0
                        const endTimeOffset = value === CONSTS.shift ? 16 : 0
                        const endDateOffset = value === CONSTS.shift ? -1 : 0
                        const startDate = source.parent.record.startDate;
                        const endDate = source.parent.record.endDate;
                        source.parent.widgetMap.startTimeField.value = DateHelper.add(DateHelper.clearTime(startDate), startOffset, "hour")
                        source.parent.widgetMap.endTimeField.value = DateHelper.add(DateHelper.clearTime(endDate), endTimeOffset, "hour")
                        source.parent.widgetMap.endDateField.value = DateHelper.add(DateHelper.clearTime(endDate), endDateOffset, "hour")
                    }
                }
            },
            nameField: {
                dataset: { eventType: CONSTS.shift },
            },
            resourceField: null,
            userField: {
                required: true,
                label: "User", // TODO: Translate..
                name: rowKey === "user_id" ? "resourceId" : "userId", // NOTE: Make sure your EventModel has this field for this to link up correctly
                type: 'combo',
                ref: 'userId',
                store: users,
                placeholder: 'Select a user...',
                weight: 130,
                valueField: 'id',
                displayField: 'name',
            },
            wishTypeField: {
                label: "Wishtype", // TODO: Translate..
                dataset: { eventType: CONSTS.wish }, // TODO: Once backend returns this as enum..
                name: "wishTypeId", // NOTE: Make sure your EventModel has this field for this to link up correctly
                type: 'combo',
                ref: 'wishTypeCombo',
                store: wishTypes,
                placeholder: 'Select wish type...',
                weight: 130,
                valueField: 'id',
                displayField: 'name',
            },
            roomField: {
                label: "Room", // TODO: Translate..
                dataset: { eventType: CONSTS.shift }, // TODO: Once backend returns this as enum..
                name: rowKey === "user_id" ? "roomId" : "resourceId", // NOTE: Make sure your EventModel has this field for this to link up correctly
                type: 'combo',
                ref: 'roomCombo',
                store: rooms,
                placeholder: 'Select room...', // TODO: Translate
                weight: 130,
                valueField: 'id',
                displayField: 'name',
            },
            startTimeField: {
                cls: 'b-inline',
                flex: '30%',
                required: true,
                validateOnInput: false,
            },
            startDateField: {
                cls: 'b-inline',
                flex: '50%',
                required: true,
                validateOnInput: false,
                picker: {
                    showWeekColumn: true,
                }
            },
            endTimeField: {
                cls: 'b-inline',
                flex: '30%',
                required: true,
                validateOnInput: false,
            },
            endDateField: {
                cls: 'b-inline',
                flex: '50%',
                required: true,
                validateOnInput: false,
                picker: {
                    showWeekColumn: true,
                },
                listeners: {
                    change: ({ source, value }) => {
                        // By default, a full day spans from 00:00 until 00:00 the next day. This works great for wishing a full day off.
                        // However, if you wish from the Monday to the Wednesday it will be until Wednesday 00:00, which is only up until Tuesday.
                        // So if selecting future dates, be sure fast-forward until 23:59:59
                        const originalEndDate = source.parent.record.endDate;
                        const futureDateIsSelected = originalEndDate < value
                        if (futureDateIsSelected) {
                            source.parent.widgetMap.endTimeField.value = DateHelper.add(DateHelper.endOf(value), -1, 'second')
                            return;
                        }
                    },
                }
            },
            noteField: {
                label: "Note", // TODO: Translate..
                name: "note", // NOTE: Make sure your EventModel has this field for this to link up correctly
                type: 'textarea',
                ref: 'noteField',
                placeholder: 'Note...',
                weight: 800,
            }
        },
    }
}